
import model.ImageRGBData;
import model.ImageRGBDataLoader;
import model.Algorithm;

import javax.swing.*;
import java.awt.*;
import java.awt.image.BufferedImage;
import java.awt.image.DataBufferInt;
import java.io.BufferedInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;

public class GraphicComponent extends JComponent implements Runnable {

    public static final int MIN_DELAY = 20; //there should be atleast 20 milliseconds delay betwn pics

    private ImageRGBData[] imageData;

    private int frameIndex = -1;
    private Thread playThread;
    private boolean pause;

    final private int width;
    final private int height;
    final private int padding;
    final private int delay;

    private long loadTime;

    BufferedImage bufferImage;
    BufferedImage bufferImage2;
    private Algorithm algorithm;


    public GraphicComponent(final int width,
                            final int height,
                            final int padding,
                            final int delay) {
        this.width = width;
        this.height = height;
        this.padding = padding;
        this.delay = delay;

        bufferImage = new BufferedImage(width, height, BufferedImage.TYPE_INT_RGB);
        bufferImage2 = new BufferedImage(width, height, BufferedImage.TYPE_INT_RGB);

        algorithm = new Algorithm(Algorithm.LOGARITHMIC, Algorithm.INTEGRAL);
    }

    public void load(File file) throws IOException {

        reset();

        final long time = System.currentTimeMillis();
        imageData = new ImageRGBData[(int) (file.length() / (width * height * 3))];

        final FileInputStream fis = new FileInputStream(file);
        final BufferedInputStream bis = new BufferedInputStream(fis);

        // used for optimization
        final int[] buffer1 = new int[width * height];  // red
        final int[] buffer2 = new int[buffer1.length];  // green

        for (int i = 0; i < imageData.length; i++) {
            imageData[i] = ImageRGBDataLoader.load(bis, width, height, buffer1, buffer2);
        }

        loadTime = System.currentTimeMillis() - time;
    }

    public void paint(Graphics g) {
        if (imageData == null || frameIndex == -1) {
            g.setColor(Color.black);
            g.fillRect(padding, padding, width, height);
            g.fillRect(width + (padding * 2), padding, width, height);
        } else if (frameIndex < imageData.length) {
            g.drawImage(bufferImage, padding, padding, this);
            g.drawImage(bufferImage2, width + (padding * 2), padding, this);
            g.setColor(Color.black);
            g.drawString("Frame: " + (frameIndex + 1)+" MV: " + algorithm.getNumMvs() +
                        " Objs: " + algorithm.getNumObjects(), padding, padding - 2);


        }
    }

    public synchronized void run() {
        for (; frameIndex < imageData.length; frameIndex++) {
            if (pause) {
                break;
            } else {

                long beg = System.currentTimeMillis();

                setupFrames();

                repaint();

                try {
                    long adjustedDelay = (delay - (System.currentTimeMillis() - beg));
                    //System.err.println("delay [" + frameIndex + "] = " + adjustedDelay);
                    if (adjustedDelay < MIN_DELAY) adjustedDelay = MIN_DELAY;

                    Thread.sleep(adjustedDelay);
                } catch (InterruptedException ie) {
                    ie.printStackTrace();
                }
            }
        }
    }

    public long getLoadTime() {
        return loadTime;
    }

    // Video actions
    public synchronized void reset() {
        pause = true;
        while (playThread != null && playThread.isAlive()) ;
        frameIndex = -1;
        setupFrames();
        repaint();
    }

    public synchronized void jumpBeg() {
        pause = true;
        while (playThread != null && playThread.isAlive()) ;
        frameIndex = 0;
        setupFrames();
        repaint();
    }

    public synchronized void rewind() {
        pause = true;
        while (playThread != null && playThread.isAlive()) ;
        if (frameIndex > 0) {
            frameIndex--;
            setupFrames();
            repaint();
        }
    }

    public synchronized void play() {
        if (playThread == null || !playThread.isAlive()) {
            if (frameIndex == imageData.length) {
                frameIndex = 0;
            }
            pause = false;
            playThread = new Thread(this);
            playThread.start();
        }
    }

    public synchronized void pause() {
        pause = true;
        while (playThread != null && playThread.isAlive()) ;
    }

    public synchronized void stop() {
        reset();
    }

    public synchronized void forward() {
        pause = true;
        while (playThread != null && playThread.isAlive()) ;
        if (frameIndex < imageData.length - 1) {
            frameIndex++;
            setupFrames();
            repaint();
        }
    }

    public synchronized void setupFrames()
    {
        if (frameIndex < 0 || frameIndex >= imageData.length) return;

        //copy the current image (loaded from file previously) into the left hand side image
        int[] rgb = ((DataBufferInt) (bufferImage).getRaster().getDataBuffer()).getData();
        System.arraycopy(imageData[frameIndex].getData(), 0, rgb, 0, imageData[frameIndex].getData().length);

        //call the algorithm to generate the right hand side image
        int[] rgb2 = ((DataBufferInt) (bufferImage2).getRaster().getDataBuffer()).getData();
        if (frameIndex == 0)
            System.arraycopy(imageData[frameIndex].getData(), 0, rgb2, 0, imageData[frameIndex].getData().length);
        else
            algorithm.runAlgo(imageData[frameIndex], imageData[frameIndex-1], rgb2);
    }

    public synchronized void jumpEnd() {
        pause = true;
        while (playThread != null && playThread.isAlive()) ;
        frameIndex = imageData.length - 1;
        repaint();
    }
}
